/*! \file kogmo_rtdb_reader.c
 * \brief Example for reading from the RTDB
 *
 * Copyright (c) 2005,2006 Matthias Goebl <mg*tum.de>
 *     Lehrstuhl fuer Realzeit-Computersysteme (RCS)
 *     Technische Universitaet Muenchen (TUM)
 */

#include <stdio.h> /* printf */
#include <unistd.h> /* getpid */
#include "kogmo_rtdb.h"

#define DIEonERR(value) if (value<0) { \
 fprintf(stderr,"%i DIED in %s line %i with error %i\n",getpid(),__FILE__,__LINE__,-value);exit(1);}

int
main (int argc, char **argv)
{
  kogmo_rtdb_handle_t *dbc;
  kogmo_rtdb_connect_info_t dbinfo;
  kogmo_rtdb_obj_info_t roadobj_info;
  kogmo_rtdb_obj_a2_roadkloth_t roadobj;
  kogmo_rtdb_objid_t oid;
  kogmo_rtdb_objsize_t olen;
  int err;
  
  // Verbindung zur Datenbank aufbauen, unsere Zykluszeit is 33 ms
  err = kogmo_rtdb_connect_initinfo (&dbinfo, "", "a2_roadviewer_example", 0.033); DIEonERR(err);
  oid = kogmo_rtdb_connect (&dbc, &dbinfo); DIEonERR(oid);

  // Auf Object fuer Fahrspur warten
  oid = kogmo_rtdb_obj_searchinfo_wait (dbc, "a2_eigenfahrspur",
    KOGMO_RTDB_OBJTYPE_A2_ROADKLOTH_V1, 0, 0); DIEonERR(oid);

  // Infoblock des Objektes holen und anzeigen
  err = kogmo_rtdb_obj_readinfo (dbc, oid, 0, &roadobj_info); DIEonERR(err);
  printf("rel K_CGP: x=%f, y=%f, z=%f, yaw=%f, pitch=%f, roll=%f\n",
    roadobj.sixdof.x, roadobj.sixdof.y, roadobj.sixdof.z,
    roadobj.sixdof.yaw, roadobj.sixdof.pitch, roadobj.sixdof.roll);

  // Objekt erstmalig holen (initialer Timestamp)
  err = kogmo_rtdb_obj_readdata_waitnext (dbc, oid, 0, &roadobj, sizeof(roadobj)); DIEonERR(err);

  while (1)
    {
      kogmo_timestamp_string_t timestring;
      olen = kogmo_rtdb_obj_readdata_waitnext (dbc, oid, roadobj.base.committed_ts, &roadobj, sizeof(roadobj)); DIEonERR(olen);
      if ( olen < sizeof(roadobj) )
        continue; // object to small (shouldn't happen - ask the object-writer!)
      kogmo_timestamp_to_string(roadobj.base.data_ts, timestring);
      printf("%s: l=%f, c0=%f, c1=%f, b0=%f, b1=%f,...\n", timestring,
        roadobj.roadkloth.l[0], roadobj.roadkloth.c0[0], roadobj.roadkloth.c1[0], roadobj.roadkloth.b0[0], roadobj.roadkloth.b1[0]);
    }

  err = kogmo_rtdb_disconnect(dbc, NULL); DIEonERR(err);

  return 0;
}
