/*! \file kogmo_rtdb_writer.c
 * \brief Example for writing to the RTDB
 *
 * Copyright (c) 2005,2006 Matthias Goebl <mg*tum.de>
 *     Lehrstuhl fuer Realzeit-Computersysteme (RCS)
 *     Technische Universitaet Muenchen (TUM)
 */

#include <stdio.h> /* printf */
#include <unistd.h> /* sleep,getpid */
#include <math.h> /* sin,cos */
#include "kogmo_rtdb.h"

#define DIEonERR(value) if (value<0) { \
 fprintf(stderr,"%i DIED in %s line %i with error %i\n",getpid(),__FILE__,__LINE__,-value);exit(1);}

int
main (int argc, char **argv)
{
  kogmo_rtdb_handle_t *dbc;
  kogmo_rtdb_connect_info_t dbinfo;
  kogmo_rtdb_obj_info_t roadobj_info;
  kogmo_rtdb_obj_a2_roadkloth_t roadobj;
  kogmo_rtdb_objid_t oid;
  int err, i, imax;
  
  // Verbindung zur Datenbank aufbauen, unsere Zykluszeit is 33 ms
  err = kogmo_rtdb_connect_initinfo (&dbinfo, "", "a2_roadtracker_example", 0.033); DIEonERR(err);
  oid = kogmo_rtdb_connect (&dbc, &dbinfo); DIEonERR(oid);

  // Object fuer Fahrspur erstellen
  err = kogmo_rtdb_obj_initinfo (dbc, &roadobj_info,
    "a2_eigenfahrspur", KOGMO_RTDB_OBJTYPE_A2_ROADKLOTH_V1, sizeof (roadobj)); DIEonERR(err);
  // roadobj_info.flags.unique=1; // falls es nur eines geben darf
  oid = kogmo_rtdb_obj_insert (dbc, &roadobj_info); DIEonERR(oid);

  // Datenobjekt initialisieren
  err = kogmo_rtdb_obj_initdata (dbc, &roadobj_info, &roadobj); DIEonERR(err);

  // Fixe Werte eintragen
  roadobj.sixdof.x     = 2.5;
  roadobj.sixdof.y     = 0;
  roadobj.sixdof.z     = -0.5;
  roadobj.sixdof.yaw   = 0;
  roadobj.sixdof.pitch = 0;
  roadobj.sixdof.roll  = 0;

  imax = 100; // demo
  for ( i=1; i<=imax; i++ )
    {
      // Von welchem Zeitpunkt sind die Daten?
      roadobj.base.data_ts = kogmo_timestamp_now();
      // Nun die eigentlichen Daten (Achtung: sinnlos! Nur zur Demo!)
      roadobj.roadkloth.n=1; // nur ein Klothoidensegment
      roadobj.roadkloth.l[0] = 1.5; // Beispieldaten, die sich aendern...
      roadobj.roadkloth.c0[0] = 10.0*i;
      roadobj.roadkloth.c1[0] = 10.0/i;
      roadobj.roadkloth.b0[0] = sin((float)i/imax);
      roadobj.roadkloth.b1[0] = cos((float)i/imax);

      // Daten schreiben
      err = kogmo_rtdb_obj_writedata (dbc, roadobj_info.oid, &roadobj); DIEonERR(err);

      sleep (1);
    }

  err = kogmo_rtdb_obj_delete (dbc, &roadobj_info); DIEonERR(err);

  err = kogmo_rtdb_disconnect (dbc, NULL); DIEonERR(err);

  return 0;
}
