/*! \file kogmo_rtdb_writer_cxx.cxx
 * \brief Example for writing to the RTDB with the C++ Interface
 *
 * Copyright (c) 2005,2006 Matthias Goebl <mg*tum.de>
 *     Lehrstuhl fuer Realzeit-Computersysteme (RCS)
 *     Technische Universitaet Muenchen (TUM)
 */

#include <iostream>
#include <cmath>
#include <unistd.h>
using namespace std;

#include "kogmo_rtdb.hxx"
using namespace KogniMobil;

int
main (int argc, char **argv)
{
try {

  // Verbindung zur Datenbank aufbauen, unsere Zykluszeit is 33 ms
  RTDBConn DBC("a2_roadtracker_example_cxx", 0.033, "");

  // Object fuer Fahrspur erstellen
  A2_RoadKloth Road(DBC, "a2_eigenfahrspur");
  Road.RTDBInsert();

  // Datenobjekt initialisieren
  Road.setSixDoF (2.5, 0, -0.5, 0, 0, 0);

  int i, imax = 100; // demo
  //for ( i=1; i<=imax; i++ )
  for ( i=1;/*endlos*/; i++ )
    {
      // Auf naechsten Zyklus warten
      DBC.WaitNextCycle();

      // Von welchem Zeitpunkt sind die Daten?
      Road.setTimestamp(DBC.getCycleTimestamp());
      // Nun die eigentlichen Daten (Achtung: sinnlos! Nur zur Demo!)
      Road.setNumKlothSegs (1); // nur ein Klothoidensegment
      Road.setKlothSeg ( 1, // Beispieldaten, die sich aendern...
        1.5, 10.0*i, 10.0/i, sin((float)i/imax), cos((float)i/imax) );

      Road.setPhi(0);
      Road.setY(0.4*sin(3.14*i/100));

      // Daten schreiben
      Road.RTDBWrite();

      // Zyklus fertig, als Lebenszeichen
      DBC.CycleDone();
    }

  return 0;
}
catch(DBError err)  
{   
  cout << "Died on Error: " << err.what() << endl;   
  return 1;
}   
};
